/**HEADER********************************************************************
* 
* Copyright (c) 2011 by Andrey Butok, Freescale Semiconductor;
* All Rights Reserved
*
*
*************************************************************************** 
*
* THIS SOFTWARE IS PROVIDED BY FREESCALE "AS IS" AND ANY EXPRESSED OR 
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
* OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  
* IN NO EVENT SHALL FREESCALE OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
* STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
* IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
* THE POSSIBILITY OF SUCH DAMAGE.
*
**************************************************************************
*
* $FileName: ip6_trace.c$
* $Version : 3.8.1.0$
* $Date    : Jun-14-2012$
*
* Comments:
*
*   This file contains the implementation of the TCP/IP tracing feature.
*   It is based on FNET project.
*
*   2011.10: Original version by Andrey Butok.
*
*END************************************************************************/

#include <rtcs.h>

#if RTCSCFG_ENABLE_IP6

#include "rtcs_prv.h"
#include "tcpip.h"
#include "ip_prv.h"

#include "icmp6_prv.h"
#include "ip6_trace.h"


/* Foreground color.*/
#define FNET_SERIAL_ESC_FG_BLACK    "\33[30m"
#define FNET_SERIAL_ESC_FG_GREEN    "\33[32m"
#define FNET_SERIAL_ESC_FG_BLUE     "\33[34m"

/************************************************************************
* NAME: fnet_mac_to_str
*
* DESCRIPTION: Converts MAC address to an null-terminated string.
*************************************************************************/
static void mac_to_str( _enet_address addr, char *str_mac )
{
    unsigned char *p;

    p = (unsigned char *)addr;
    sprintf(str_mac, "%02X:%02X:%02X:%02X:%02X:%02X", p[0], p[1], p[2],
                        p[3], p[4], p[5]);
}

/************************************************************************
* NAME: fnet_mac_to_str
*
* DESCRIPTION: Converts MAC address to an null-terminated string.
*************************************************************************/
static void ip6_to_str( char *addr, char *str_addr )
{
    unsigned char *p;

    p = (unsigned char *)addr;
    sprintf(str_addr," %02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x ", 
    ((uint_8 *)addr)[0], ((uint_8 *)addr)[1], ((uint_8 *)addr)[2], ((uint_8 *)addr)[3], 
    ((uint_8 *)addr)[4], ((uint_8 *)addr)[5], ((uint_8 *)addr)[6], ((uint_8 *)addr)[7], 
    ((uint_8 *)addr)[8], ((uint_8 *)addr)[9], ((uint_8 *)addr)[10], ((uint_8 *)addr)[11], 
    ((uint_8 *)addr)[12], ((uint_8 *)addr)[13], ((uint_8 *)addr)[14], ((uint_8 *)addr)[15]);
}

/************************************************************************
* NAME: fnet_eth_trace
*
* DESCRIPTION: Prints an Ethernet header. For debug needs only.
*************************************************************************/
#if RTCSCFG_DEBUG_TRACE_ETH
void trace_eth(char *str, ENET_HEADER_PTR eth_hdr)
{
    char mac_str[18];

    printf(FNET_SERIAL_ESC_FG_GREEN"%s", str); /* Print app-specific header.*/
    printf("[ETH header]\n"FNET_SERIAL_ESC_FG_BLACK); 
    printf("+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+/\\/\\/\\/-+\n");
    mac_to_str(eth_hdr->DEST, mac_str);
    printf("|(Dest)                                                "FNET_SERIAL_ESC_FG_BLUE"%17s"FNET_SERIAL_ESC_FG_BLACK" |\n", mac_str);
    printf("+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+/\\/\\/\\/-+\n");
    mac_to_str(eth_hdr->SOURCE, mac_str);
    printf("|(Src)                                                 "FNET_SERIAL_ESC_FG_BLUE"%17s"FNET_SERIAL_ESC_FG_BLACK" |\n", mac_str);
    printf("+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+/\\/\\/\\/-+\n");
    printf("|(Type)                  0x%02x%02x |\n", eth_hdr->TYPE[0], eth_hdr->TYPE[1]/*fnet_ntohs(eth_hdr->type)*/);
    printf("+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+\n");
}
#endif /* RTCSCFG_DEBUG_TRACE_ETH */

/************************************************************************
* NAME: trace_ip6
*
* DESCRIPTION: Prints an IP6 header. For debug needs only.
*************************************************************************/
#if RTCSCFG_DEBUG_TRACE_IP6
void trace_ip6(char *str, IP6_HEADER_PTR ip_hdr)
{
    char addr_str[40];

    printf(FNET_SERIAL_ESC_FG_GREEN"%s", str); /* Print app-specific header.*/
    printf("[IP6 header]\n"FNET_SERIAL_ESC_FG_BLACK); //TBD Add Timestamp
    printf("+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+\n");
    printf("|(V) %2d |(TClass)  0x%02x |(FlowLabel)                    0x%05x |\n",
                    (ip_hdr->VERSION_TCLASS[0] & 0xF0)>>4,
                    ((ip_hdr->VERSION_TCLASS[0] & 0x0F)<<4) + ((ip_hdr->TCLASS_FLOWL[0] & 0xF0)>>4),
                    ((ip_hdr->TCLASS_FLOWL[0] & 0xF0)<<16) + ntohs(ip_hdr->FLOWL));
    printf("+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+\n");
    printf("|(PLength)                %5u |(Protocol) "FNET_SERIAL_ESC_FG_BLUE"%3u"FNET_SERIAL_ESC_FG_BLACK" |(TTL)      %3u |\n",
                    ntohs(ip_hdr->LENGTH),
                    ip_hdr->NEXT_HEADER[0],
                    ip_hdr->HOP_LIMIT[0]);
    printf("+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+/\\/\\/\\/-+\n");
    ip6_to_str((char*)ip_hdr->SOURCE, addr_str);
    printf("|(Src)                         "FNET_SERIAL_ESC_FG_BLUE"%39s"FNET_SERIAL_ESC_FG_BLACK" |\n",
                     addr_str);
    printf("+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+/\\/\\/\\/-+\n");                    
    ip6_to_str((char*)ip_hdr->DEST, addr_str);
    printf("|(Dest)                        "FNET_SERIAL_ESC_FG_BLUE"%39s"FNET_SERIAL_ESC_FG_BLACK" |\n",
                     addr_str);
    printf("+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+/\\/\\/\\/-+\n");     
}
#endif /* RTCSCFG_DEBUG_TRACE_IP6 */


/************************************************************************
* NAME: fnet_icmp6_trace
*
* DESCRIPTION: Prints an ICMP6 header. For debug needs only.
*************************************************************************/
#if RTCSCFG_DEBUG_TRACE_ICMP6
void trace_icmp6(char *str, ICMP6_HEADER_PTR icmp_hdr)
{

    printf(FNET_SERIAL_ESC_FG_GREEN"%s", str); /* Print app-specific header.*/
    printf("[ICMP6 header]\n"FNET_SERIAL_ESC_FG_BLACK);
    printf("+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+\n");
    printf("|(Type)     "FNET_SERIAL_ESC_FG_BLUE"%3u"FNET_SERIAL_ESC_FG_BLACK" |(Code)     %3u |(Cheksum)               0x%04x |\n",
                    icmp_hdr->TYPE[0],
                    icmp_hdr->CODE[0],
                    ntohs(icmp_hdr->CHECKSUM));
    printf("+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+\n");
    
}
#endif /* RTCSCFG_DEBUG_TRACE_ICMP6 */

#endif /* RTCSCFG_ENABLE_IP6 */
